/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.api.abstractElmt;

import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.logging.Logger;

import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.SchemaReader;
import org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants;
import org.ow2.easywsdl.schema.api.absItf.AbsItfInclude;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.schema.impl.Constants;
import org.ow2.easywsdl.schema.impl.SchemaReaderImpl;
import org.ow2.easywsdl.schema.util.Util;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public abstract class AbstractIncludeImpl<E, S extends AbsItfSchema> extends AbstractSchemaElementImpl<E> implements AbsItfInclude<S> {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(AbstractIncludeImpl.class.getName());

	/**
	 * the parent
	 */
	protected S schema;

	/**
	 * Default constructor
	 * 
	 * @param model
	 *            the model
	 * @param parent
	 *            the parent description
	 */
	@SuppressWarnings("unchecked")
	public AbstractIncludeImpl(final String baseURIString, final E model, final Map<FeatureConstants, Object> features, final Map<URI, AbsItfSchema> imports, SchemaReaderImpl reader) throws SchemaException {
		super(model, null);

		String location = this.getLocationURI();

		if (location != null && !Util.isURL(location) && baseURIString != null) {
			location = baseURIString + location;
		}

		if (features != null) {
			final Object obj = features.get(FeatureConstants.IMPORT_DOCUMENTS);
			if ((Boolean) obj) {
				this.retrieveInclude(location, features, imports, reader);
			}
		}

		if (this.schema != null) {
			((AbstractSchemaImpl) this.schema).setFeatures(features);
		}

	}

	/**
	 * Default constructor
	 * 
	 * @param model
	 *            the model
	 * @param parent
	 *            the parent description
	 */
	@SuppressWarnings("unchecked")
	public AbstractIncludeImpl(final E model, final AbstractSchemaElementImpl parent, final Map<URI, AbsItfSchema> imports, SchemaReaderImpl reader) throws SchemaException {
		super(model, parent);
		try {
			String location = this.getLocationURI();
			if(location != null) {
				location = location.trim();
			}

			if (location != null && !Util.isURL(location) && parent.getSchema().getDocumentBaseURIString() != null) {
				location = parent.getSchema().getDocumentBaseURIString() + location;
				URI parentURI = parent.getSchema().getDocumentURI();
				if((!"file".equals(parentURI.getScheme())) && (!"jar".equals(parentURI.getScheme())) && (!"classpath".equals(parentURI.getScheme())) && parentURI.getScheme() != null) {
					if(parentURI.getPort() != -1) {
						location = parentURI.getScheme() + "://" + parentURI.getHost() + ":" + parentURI.getPort() + location;
					} else {
						location = parentURI.getScheme() + "://" + parentURI.getHost() + location;
					}
				}
			}

			if (this.parent != null) {
				if (((AbstractSchemaImpl) this.parent).getFeatures() != null) {
					if ((((AbstractSchemaImpl) this.parent).getFeatures().get(FeatureConstants.IMPORT_DOCUMENTS) != null) && ((Boolean) ((AbstractSchemaImpl) this.parent).getFeatures().get(FeatureConstants.IMPORT_DOCUMENTS))) {
						this.retrieveInclude(location, ((AbstractSchemaImpl) this.parent).getFeatures(), imports, reader);
					} else if(imports != null && !imports.isEmpty() && imports.get(new URI(location)) != null) {
						this.schema =  (S) imports.get(new URI(location));
					}
				} else if(imports != null && !imports.isEmpty() && imports.get(new URI(location)) != null) {
					this.schema =  (S) imports.get(new URI(location));
				}
			}

			if ((parent != null) && (this.schema != null)) {
				((AbstractSchemaImpl) this.schema).setFeatures(((AbstractSchemaImpl) this.parent).getFeatures());
			}
		} catch (URISyntaxException e) {
			throw new SchemaException(e);
		}
	}

	private void retrieveInclude(final String location, final Map<FeatureConstants, Object> features, final Map<URI, AbsItfSchema> imports, SchemaReaderImpl reader) throws SchemaException {
		InputStream is = null;

		if (location != null) {
			try {
				URI loc = null;

				// Check for well known schemas
				if (Constants.XML_URI.equals(location)) {
					loc = new URI(getClass().getClassLoader().getResource(Constants.XML_XSD).toString().replaceAll(" ", "%20"));
				} else {
					loc = new URI(location);
				}

				String locationNorm = loc.normalize().toString();

				// Try to identify a cyclic import loop
				if(reader.getImportList().containsKey(locationNorm)) {
					log.warning("This import is already include : " + locationNorm + ". This probably mean there's a cyclic import");
					this.schema = (S) reader.getImportList().get(locationNorm);
				} else {
					reader.getImportList().put(locationNorm, null);

					if (imports != null && imports.containsKey(loc)) {
						// Search in imports Map
						this.schema = (S) imports.get(loc);
					} else {
						// Create schema reader
						reader.setFeatures(features);
						this.schema = (S) ((SchemaReaderImpl)reader).readSchema(loc, imports, false);
					}

				}
				reader.getImportList().put(locationNorm, this.schema);
			} catch (final SchemaException e) {
				throw new SchemaException("the imported document cannot be import at: " + location, e);
			} catch (URISyntaxException e) {
				throw new SchemaException("the imported document cannot be import at: " + location + " because the URI is invalid", e);
			}
		}
	}

	/**
	 * @return the parent
	 */
	public S getSchema() {
		return this.schema;
	}

	public void setSchema(final S schema) {
		this.schema = schema;
	}

	/**
	 * @return the parent parent
	 */
	public S getParentSchema() {
		return (S) this.parent.getSchema();
	}

}
